@@ -6,13 +6,25 @@ import re
 import glob
 import filecmp
 import difflib
-from ami_val.libs.utils_lib import run_cmd, is_arch, get_product_id, is_fedora, is_cmd_exist
+from ami_val.libs.utils_lib import run_cmd, is_arch, get_product_id, is_fedora, is_cmd_exist, getboottime
 
 def test_stage1_check_bash_history(test_instance):
     for user in ['ec2-user', 'root']:
         cmd = 'sudo cat ~{}/.bash_history'.format(user)
         run_cmd(test_instance, cmd, expect_not_ret='0', msg='check bash history does not exist in fresh AMI')
 
+def test_stage1_check_bootime_launch(test_instance):
+    '''
+    check first launch boot time.
+    can change pass criteria in cfg ami-val.yaml, default is 50s.
+    rhbz: 1862930
+    '''
+
+    boottime = getboottime(test_instance)
+    if float(boottime) > float(test_instance.params['max_boot_time']):
+        test_instance.fail('boot time {} over expected {}'.format(boottime, test_instance.params['max_boot_time']))
+    else:
+        test_instance.log.info('boot time {} within expected {}'.format(boottime, test_instance.params['max_boot_time']))
 
 def test_stage1_check_cds_hostnames(test_instance):
     '''
@@ -60,6 +72,14 @@ def test_stage1_check_chrony_aws(test_instance):
     rhbz: 1679763 [RFE] AWS AMI - Add Amazon Timesync Service
     '''
     run_cmd(test_instance, "sudo cat /etc/chrony.conf", expect_ret=0, expect_kw='server 169.254.169.123', msg='check chrony points to Amazon Time Sync service')
+    # https://access.redhat.com/solutions/5132071
+    # https://bugzilla.redhat.com/show_bug.cgi?id=1961156
+    product_id = get_product_id(test_instance)
+    if float(product_id) < float('8.5'):
+        run_cmd(test_instance, "sudo cat /etc/chrony.conf", expect_ret=0, expect_kw="#leapsectz right/UTC,#pool 2.rhel.pool.ntp.org iburst", msg='check no NTP leap smear incompatibilitye')
+    else:
+        run_cmd(test_instance, "sudo cat /etc/chrony.conf", expect_ret=0, expect_not_kw="leapsectz right/UTC,pool 2.rhel.pool.ntp.org iburst", msg='check no NTP leap smear incompatibilitye')
+    run_cmd(test_instance, "sudo journalctl -u chronyd", expect_ret=0, expect_kw='Selected source 169.254.169.123', msg='check Amazon Time Sync service is in use')
 
 def test_stage1_check_cloud_firstboot(test_instance):
     '''
@@ -79,11 +99,20 @@ def test_stage1_check_cmdline_crashkernel(test_instance):
     crashkernel should be enabled in image
     '''
     aminame = test_instance.info['name']
+    product_id = get_product_id(test_instance)
     if 'RHEL-6' in aminame:
         run_cmd(test_instance, "sudo service kdump status")
         run_cmd(test_instance, "sudo cat /proc/cmdline", expect_ret=0, expect_not_kw='crashkernel', msg='crashkernel not required as xen kdump unsupported')
     else:
-        run_cmd(test_instance, "sudo cat /proc/cmdline", expect_ret=0, expect_kw='crashkernel=auto', msg='check crashkernel is enabled')
+        if float(product_id) < float('9'):
+            expect_kw = 'crashkernel=auto'
+        else:
+            # rhbz: 1942398
+            if is_arch(test_instance, arch='x86_64'):
+                expect_kw = 'crashkernel=1G-4G:192M,4G-64G:256M,64G-:512M'
+            else:
+                expect_kw = 'crashkernel=2G-:448M'
+        run_cmd(test_instance, "sudo cat /proc/cmdline", expect_ret=0, expect_kw=expect_kw, msg='check crashkernel is enabled')
 
 def test_stage1_check_cmdline_ifnames(test_instance):
     '''
@@ -132,6 +161,13 @@ def test_stage1_check_cmdline_nouveau(test_instance):
     if 'RHEL-6' in aminame:
         test_instance.skipTest('not required in el6')
     run_cmd(test_instance, "sudo cat /proc/cmdline", expect_ret=0, expect_kw='rd.blacklist=nouveau', msg='check nouveau is in blacklist')
+    product_id = get_product_id(test_instance)
+    if float(product_id) < float('8.5'):
+        file_check = '/etc/modprobe.d/blacklist-nouveau.conf'
+    else:
+        file_check = '/usr/lib/modprobe.d/blacklist-nouveau.conf'
+    expect_kw='blacklist nouveau'
+    run_cmd(test_instance, "sudo cat {}".format(file_check), expect_ret=0, expect_kw=expect_kw, msg='check "{}" in {}'.format(expect_kw, file_check))
 
 def test_stage1_check_cmdline_nvme_io_timeout(test_instance):
     '''
@@ -177,20 +213,39 @@ def test_stage1_check_dracut_conf_sgdisk(test_instance):
     cmd = "rpm -q gdisk"
     run_cmd(test_instance, cmd, expect_ret=0, msg='Check if gdisk is installed')
 
-    cmd = "sudo cat /etc/dracut.conf.d/sgdisk.conf"
-    run_cmd(test_instance, cmd, expect_ret=0, expect_kw='install_items+=" sgdisk "', msg='check if sgdisk is added into amis')
+    product_id = get_product_id(test_instance)
+    if float(product_id) < float('8.5'):
+        file_check = '/etc/dracut.conf.d/sgdisk.conf'
+    else:
+        file_check = '/usr/lib/dracut/dracut.conf.d/sgdisk.conf'
+    expect_kw='install_items+=" sgdisk "'
+    cmd = "sudo cat {}".format(file_check)
+    run_cmd(test_instance, cmd, expect_ret=0, expect_kw=expect_kw, msg='check if "{}" is added into {}'.format(expect_kw,file_check))
 
 def test_stage1_check_dracut_conf_xen(test_instance):
     '''
     rhbz: 1849082
-    des: add ' xen-netfront xen-blkfront ' to '/etc/dracut.conf.d/xen.conf' in x86 AMIs
-         and not required in arm AMIs
+    des: add ' xen-netfront xen-blkfront ' to '/etc/dracut.conf.d/xen.conf' in x86 AMIs prior RHEL-8.4.
+         using image builder from RHEL-8.5, updated to add ' nvme xen-blkfront ' to '/usr/lib/dracut/dracut.conf.d/ec2.conf'.
+         This is not required in arm AMIs and RHEL-7.
     '''
     cmd = "sudo cat /etc/dracut.conf.d/xen.conf"
-    if is_arch(test_instance, arch='x86_64'):
-        run_cmd(test_instance, cmd, expect_ret=0, expect_kw=' xen-netfront xen-blkfront ', msg='check if xen-netfront and xen-blkfront are added into x86 amis')
+    product_id = get_product_id(test_instance)
+    if float(product_id) < float('8'):
+        file_check = '/etc/dracut.conf.d/xen.conf'
+        test_instance.skipTest('{} is not required in el7'.format(file_check))
+    elif float(product_id) < float('8.5'):
+        file_check = '/etc/dracut.conf.d/xen.conf'
+        expect_kw=' xen-netfront xen-blkfront '
     else:
-        run_cmd(test_instance, cmd, expect_not_ret=0, msg='check no /etc/dracut.conf.d/xen.conf in arm amis')
+        #COMPOSER-1096
+        file_check = '/usr/lib/dracut/dracut.conf.d/ec2.conf'
+        expect_kw=' nvme xen-blkfront '
+    cmd = "sudo cat {}".format(file_check)
+    if is_arch(test_instance, arch='x86_64'):
+        run_cmd(test_instance, cmd, expect_ret=0, expect_kw=expect_kw, msg='check if "{}" exists in x86 amis'.format(expect_kw))
+    else:
+        run_cmd(test_instance, cmd, expect_not_ret=0, msg='check no {} in arm amis'.format(file_check))
 
 def test_stage1_check_ena_set_in_image(test_instance):
     '''
@@ -224,21 +279,46 @@ def test_stage1_check_firewalld(test_instance):
 
 def test_stage1_check_grub(test_instance):
     '''
-    Check grub config:
+    Check grub config for el5,el6:
     - /boot/grub/menu.lst exists
     - /boot/grub/menu.lst is symlink for /boot/grub/grub.conf
     - hard drive is not (hd0,0) for paravirtual
+    Check grub2 config for el8:
+    - boot with efi or legacy bios
+    - /boot/grub2/grubenv is a symlink for /boot/efi/EFI/redhat/grubenv if boot with efi
+    - /boot/grub2/grubenv is a file rather than a link if boot with legacy bios
+    Check grub2 config for el9:
+    - boot with efi or legacy bios
+    - /boot/grub2/grub.cfg exists and /boot/grub2/grubenv is a file if boot with efi
+    - /boot/grub2/grubenv is a file rather than a link if boot with legacy bios
     '''
-    aminame = test_instance.info['name']
-    if not aminame.startswith(('RHEL-6')):
-        test_instance.skipTest('only run in el5, el6')
-    cmd = 'sudo readlink -e /boot/grub/menu.lst'
-    run_cmd(test_instance, cmd, expect_ret=0, expect_kw="/boot/grub/grub.conf", 
-        msg='check /boot/grub/menu.lst is symlink for /boot/grub/grub.conf')
-    cmd = 'sudo cat /boot/grub/grub.conf'
-    out = run_cmd(test_instance, cmd, expect_ret=0, msg='get grub.conf')
-    if r"hd0,0" not in out:
-        test_instance.fail("'hd0,0' not found in grub.conf")
+    product_id = get_product_id(test_instance)
+    if float(product_id) < float('7'):
+        cmd = 'sudo readlink -e /boot/grub/menu.lst'
+        run_cmd(test_instance, cmd, expect_ret=0, expect_kw="/boot/grub/grub.conf",
+            msg='check /boot/grub/menu.lst is symlink for /boot/grub/grub.conf')
+        cmd = 'sudo cat /boot/grub/grub.conf'
+        out = run_cmd(test_instance, cmd, expect_ret=0, msg='get grub.conf')
+        if r"hd0,0" not in out:
+            test_instance.fail("'hd0,0' not found in grub.conf")
+    elif float(product_id) > float('7.9'):
+        cmd = 'sudo ls /sys/firmware/efi'
+        out = run_cmd(test_instance, cmd, msg='check if boot with efi')
+        cmd = 'sudo readlink -e /boot/grub2/grubenv'
+        if out:
+            if float(product_id) < float('9'):
+                run_cmd(test_instance, cmd, expect_ret=0, expect_kw="/boot/efi/EFI/redhat/grubenv",
+                        msg='check /boot/grub2/grubenv is symlink for /boot/efi/EFI/redhat/grubenv')
+            else:
+                run_cmd(test_instance, cmd, expect_ret=0, expect_kw="/boot/grub2/grubenv",
+                        msg='check /boot/grub2/grubenv is a file rather than a link')
+                run_cmd(test_instance, 'sudo ls /boot/grub2', expect_ret=0, expect_kw="grub.cfg",
+                        msg='check for grub.cfg')
+        else:
+            run_cmd(test_instance, cmd, expect_ret=0, expect_kw="/boot/grub2/grubenv",
+                msg='check /boot/grub2/grubenv is a file rather than a link')
+    else:
+        test_instance.skipTest('only run in el5, el6, el8, el8+')
 
 def test_stage1_check_hosts(test_instance):
     '''
@@ -248,6 +328,16 @@ def test_stage1_check_hosts(test_instance):
     cmd = "sudo cat /etc/hosts"
     run_cmd(test_instance, cmd, expect_ret=0, expect_kw=expect_kws, msg='check /etc/hosts')
 
+def test_stage1_check_hostkey_permissions(test_instance):
+    '''
+    bz:2013644
+    check ssh files permission set are correct.
+    '''
+    run_cmd(test_instance,
+            "ls -l /etc/ssh/{ssh_host_ecdsa_key,ssh_host_ed25519_key,ssh_host_rsa_key}",
+            expect_not_kw='-rw-------. 1 root root',
+            msg='check ssh files permission set are correct')
+
 def test_stage1_check_inittab(test_instance):
     '''
     check default runlevel or systemd target
@@ -281,6 +371,9 @@ def test_stage1_check_ks_file_asave(test_instance):
         test_instance.log.info('RHEL AMI found')
         ks_type = 'vanilla'
 
+    product_id = get_product_id(test_instance)
+    if float(product_id) > float('8.4'):
+        test_instance.skipTest('skip it after el8.4')
     ks_file = "{}/debug/{}_{}_{}_{}_ks.cfg".format(test_instance.logdir, test_instance.info["ami"], test_instance.info["region"], test_instance.info["release"]["arch"], ks_type)
     run_cmd(test_instance, 'sudo cp /root/anaconda-ks.cfg /tmp', expect_ret=0, msg='copy /root/anaconda-ks.cfg to /tmp')
     run_cmd(test_instance, 'sudo chmod 666 /tmp/anaconda-ks.cfg', expect_ret=0, msg='change permission')
@@ -306,6 +399,9 @@ def test_stage1_check_ks_file_compare(test_instance):
     else:
         test_instance.log.info('RHEL AMI found')
         ks_type = 'vanilla'
+    product_id = get_product_id(test_instance)
+    if float(product_id) > float('8.4'):
+        test_instance.skipTest('skip it after el8.4')
     src_ks_file = "{}/debug/{}_{}_{}_{}_ks.cfg".format(test_instance.logdir, test_instance.info["ami"], test_instance.info["region"], test_instance.info["release"]["arch"],ks_type)
     ks_match = "{}/debug/*{}_{}_ks.cfg".format(test_instance.logdir, test_instance.info["release"]["arch"], ks_type)
     test_instance.log.info("Try to match: {}".format(ks_match))
@@ -330,7 +426,7 @@ def test_stage1_check_instance_identity(test_instance):
     try to fetch instance identity from EC2 and compare with expectation
     '''
     aminame = test_instance.info['name']
-    cmd = 'curl http://169.254.169.254/latest/dynamic/instance-identity/document'
+    cmd = 'curl -s http://169.254.169.254/latest/dynamic/instance-identity/document'
     output = run_cmd(test_instance, cmd, expect_ret=0, msg='get instance identity data')
     instance = json.loads(output)
     if instance['imageId'] != test_instance.info['ami']:
@@ -358,6 +454,8 @@ def test_stage1_check_instance_identity(test_instance):
     elif 'Access2' in aminame:
         # Cloud Access billing code, means don't charge for the OS (so it can apply to anything cloud Access)
         billingcodes = ['bp-63a5400a']
+    else:
+        test_instance.skipTest('unable to decide billingcodes as no "Hourly2" and "Access2" found in AMI name')
     for billingcode in billingcodes:
         if billingcode not in instance['billingProducts']:
             test_instance.fail("expected({}) not found in instance billingcode({})".format(billingcode, instance['billingProducts']))
@@ -398,7 +496,7 @@ def test_stage1_check_network_ipv6(test_instance):
     '''
     check for networking setup
     '''
-    cmd = "curl http://169.254.169.254/latest/meta-data/network/interfaces/macs"
+    cmd = "curl -s http://169.254.169.254/latest/meta-data/network/interfaces/macs"
     mac = run_cmd(test_instance, cmd, expect_ret=0, msg='get mac address')
     cmd = "{}/{}/ipv6s".format(cmd, mac)
     ipv6s = run_cmd(test_instance, cmd, msg='get ipv6 address')
@@ -428,8 +526,16 @@ def test_stage1_check_nm_cloud_setup(test_instance):
     run_cmd(test_instance, cmd, expect_ret=0, msg='Check if NetworkManager-cloud-setup is installed')
     cmd = "sudo systemctl is-enabled nm-cloud-setup"
     run_cmd(test_instance, cmd, expect_ret=0, msg='Check if nm-cloud-setup is enabled')
-    cmd = "sudo cat /etc/systemd/system/nm-cloud-setup.service.d/10-rh-enable-for-ec2.conf"
-    run_cmd(test_instance, cmd, expect_kw='Environment=NM_CLOUD_SETUP_EC2=yes', msg='Check if NM_CLOUD_SETUP_EC2 is set')
+
+    product_id = get_product_id(test_instance)
+    #COMPOSER-842
+    if float(product_id) < float('8.5'):
+        file_check = '/etc/systemd/system/nm-cloud-setup.service.d/10-rh-enable-for-ec2.conf'
+    else:
+        file_check = '/usr/lib/systemd/system/nm-cloud-setup.service.d/10-rh-enable-for-ec2.conf'
+    expect_kw='Environment=NM_CLOUD_SETUP_EC2=yes'
+    cmd = "sudo cat {}".format(file_check)
+    run_cmd(test_instance, cmd, expect_kw=expect_kw, msg='Check if "{}" is set in {}'.format(expect_kw, file_check))
 
 def test_stage1_check_no_avc_denials(test_instance):
     '''
@@ -491,7 +597,7 @@ libertas-usb8388-firmware,firewalld,biosdevname,plymouth,iprutils'''.split(',')
     if 'SAP' in test_instance.info['name']:
         pkgs_unwanted.remove('alsa-lib')
     if 'HA' in test_instance.info['name']:
-        pkgs_unwanted.remove('rh-amazon-rhui-client')
+        pkgs_unwanted.append('rh-amazon-rhui-client')
     for pkg in pkgs_unwanted:
         cmd = 'rpm -q {}'.format(pkg)
         run_cmd(test_instance, cmd, expect_not_ret=0, msg='check {} not installed'.format(pkg))
@@ -527,6 +633,10 @@ gdisk,insights-client,dracut-config-generic,dracut-config-rescue,grub2-tools'''.
         pkgs_wanted.extend(['numactl', 'PackageKit-gtk3-module', 'xorg-x11-xauth', 'libnsl'])
         #BZ:1959962
         pkgs_wanted.append('tuned-profiles-sap-hana')
+    if float(product_id) < float('8'):
+        #For RHEL-7
+        pkgs_wanted = '''kernel,yum-utils,cloud-init,dracut-config-generic,dracut-norescue,grub2,tar,rsync,chrony'''.split(',')
+        pkgs_wanted = [ x.strip('\n') for x in pkgs_wanted ]
     for pkg in pkgs_wanted:
         cmd = 'rpm -q {}'.format(pkg)
         run_cmd(test_instance, cmd, expect_ret=0, msg='check {} installed'.format(pkg))
@@ -635,10 +745,21 @@ def test_stage1_check_timezone(test_instance):
 def test_stage1_check_udev_kernel(test_instance):
     '''
     des: /etc/udev/rules.d/80-net-name-slot.rules link to /dev/null
+    ks ref:
+    # For cloud images, 'eth0' _is_ the predictable device name, since
+    # we don't want to be tied to specific virtual (!) hardware
+    rm -f /etc/udev/rules.d/70*
+    ln -s /dev/null /etc/udev/rules.d/80-net-name-slot.rules
     '''
-    expect_kws = '/dev/null'
-    cmd = "sudo ls -l /etc/udev/rules.d/80-net-name-slot.rules"
-    run_cmd(test_instance, cmd, expect_ret=0, expect_kw=expect_kws, msg='check /etc/udev/rules.d/80-net-name-slot.rules')
+    product_id = get_product_id(test_instance)
+    if float(product_id) > float('8.4'):
+        #COMPOSER-844 - this set not required in rhel8+
+        cmd = "sudo ls -l /etc/udev/rules.d/80-net-name-slot.rules"
+        run_cmd(test_instance, cmd, expect_not_ret=0, msg='check /etc/udev/rules.d/80-net-name-slot.rules not existing')
+    else:
+        expect_kws = '/dev/null'
+        cmd = "sudo ls -l /etc/udev/rules.d/80-net-name-slot.rules"
+        run_cmd(test_instance, cmd, expect_ret=0, expect_kw=expect_kws, msg='check /etc/udev/rules.d/80-net-name-slot.rules')
     cmd = 'sudo cat /etc/udev/rules.d/70-persistent-net.rules'
     run_cmd(test_instance, cmd, expect_ret=0,msg='check /etc/udev/rules.d/70-persistent-net.rules')
 
