@@ -3,7 +3,7 @@ import re
 import sys
 import time
 import json
-from ami_val.libs.utils_lib import run_cmd, get_product_id
+from ami_val.libs.utils_lib import run_cmd, get_product_id, getboottime
 import ami_val
 
 def test_stage2_check_auditd(test_instance):
@@ -111,6 +111,18 @@ def test_stage2_check_ttyS0_conf(test_instance):
     run_cmd(test_instance, 'sudo cat /etc/init/ttyS0.conf', expect_not_ret=0, msg='make sure no /etc/init/ttyS0.conf found')
     run_cmd(test_instance, 'sudo cat /etc/init/ttyS0.bak', msg='ttyS0.bak may also not in RHEL nowadays')
 
+def test_stage2_test_reboot_grubby(test_instance):
+    '''
+    check user can update boot parameter using grubby tool
+    '''
+
+    run_cmd(test_instance, 'sudo grubby --update-kernel=ALL --args="kmemleak=on"', msg='append boot parameter')
+    test_instance.ssh_client.close()
+    test_instance.vm.reboot()
+    test_instance.ssh_client = test_instance.vm.new_ssh_client()
+    run_cmd(test_instance, 'sudo cat /proc/cmdline', expect_kw="kmemleak=on", msg='check if the parameter has been added')
+    run_cmd(test_instance, 'sudo grubby --update-kernel=ALL --remove-args="kmemleak=on"', msg='remove the appended boot parameter')
+
 def test_stage2_test_reboot_hostname(test_instance):
     '''
     check that reboot doesn't change the hostname
@@ -122,4 +134,20 @@ def test_stage2_test_reboot_hostname(test_instance):
     test_instance.ssh_client = test_instance.vm.new_ssh_client()
     run_cmd(test_instance, 'last', expect_ret=0, msg='get last history')
     run_cmd(test_instance, 'hostname', expect_ret=0, expect_kw=hostname_1, msg='check hostname after reboot')
+
+def test_stage2_test_rebootime(test_instance):
+    '''
+    rhbz: 1776710, 1446698, 1446688
+    check reboot time after 1st init.
+    can change pass criteria in cfg ami-val.yaml, default is 30s.
+    '''
+
+    test_instance.ssh_client.close()
+    test_instance.vm.reboot()
+    test_instance.ssh_client = test_instance.vm.new_ssh_client()
+    boottime = getboottime(test_instance)
+    if float(boottime) > float(test_instance.params['max_reboot_time']):
+        test_instance.fail('boot time {} over expected {}'.format(boottime, test_instance.params['max_reboot_time']))
+    else:
+        test_instance.log.info('boot time {} within expected {}'.format(boottime, test_instance.params['max_reboot_time']))
         
\ No newline at end of file
